//
//  Motor.cpp
//  Index
//
//  Created by Takuji on 2018/08/14.
//  Copyright © 2018年 Takuji . All rights reserved.
//

#include "MotorDriver.hpp"

MotorDriver::MotorDriver(SPI* spi_module,PinName CSpin){
    MD = spi_module;
    MD->format(8, 0);
    MD->frequency(25000);

    CS = new DigitalOut(CSpin);
    CS->write(1);
    
//    comm = new Ticker();
//    comm->attach(this,&MotorDriver::communication,0.05f);
}

void MotorDriver::communication()
{
    CS->write(0);     //start communication
    wait_us(200);
    
    //1st byte, mode
    m_driver_mode = MD->write(m_mode);
    wait_us(100);
    
    //2nd byte, pow
    m_strength = max(min(m_strength,200),0);
    m_driver_current = (float)MD->write(m_strength)/10;
    wait_us(100);
    
    //3rd byte, random number
    int random_number = rand()%0xff;
    if(random_number == m_last_sent_random_num)
        random_number = rand()%0xff;
    m_last_sent_random_num = random_number;
    int driver_rand_num = MD->write(random_number);
    wait_us(100);

    //4byte checksum
    int checksum = MD->write((m_mode+m_strength+random_number)%0xff);
    wait_us(100);
    
    CS->write(1);     //end communication
    
    
//    if((m_driver_mode + m_driver_current*10 + driver_rand_num)%0xff != checksum){
//        m_driver_current = -1;
//        return -1;
//    }
//    else if(driver_rand_num == m_last_received_random_num){
//        m_driver_current = -1;
//        return -1;
//    }
//    else
//        return 0;
    
    m_last_received_random_num = driver_rand_num;
}


int MotorDriver::write(float duty)
{
    //duty比で-1から1のデータ
    if(duty>0)
        m_mode = CW_DUTY;
    if(duty<0)
        m_mode = CCW_DUTY;
    else
        m_mode = FREE;
    m_strength = abs(duty*250);
    
    //通信のステータスを返り値に
//    if(!MotorDriver::communication())
//        return MotorDriver::communication();
//    else
//        return 0;
}


int MotorDriver::writeCurrent(float current)
{
    //duty比で-1から1のデータ
    if(current>0)
        m_mode = CW_CURRENT;
    if(current<0)
        m_mode = CCW_CURRENT;
    else
        m_mode = FREE;
    m_strength = abs((char)(current*10));
    
    //通信のステータスを返り値に
//    if(!MotorDriver::communication())
//        return MotorDriver::communication();
//    else
//        return 0;
}


int MotorDriver::brake()
{
    m_mode = BRAKE;
    m_strength = 0;

    //通信のステータスを返り値に
//    if(!MotorDriver::communication())
//        return MotorDriver::communication();
//    else
//        return 0;
}

int MotorDriver::brake(float duty)
{
    m_mode = PWM_BRAKE;
    m_strength = duty*200;
    
    //通信のステータスを返り値に
//    if(!MotorDriver::communication())
//        return MotorDriver::communication();
//    else
//        return 0;
}


int MotorDriver::stop()
{
    m_mode = FREE;
    m_strength = 0;
    
    //通信のステータスを返り値に
//    if(!MotorDriver::communication())
//        return MotorDriver::communication();
//    else
//        return 0;
}


float MotorDriver::getCurrent()
{
    return m_driver_current;
}
